@@ -25,6 +25,7 @@
 #include "rpc/local-repo.h"
 #include "rpc/rpc-client.h"
 #include "ui/private-share-dialog.h"
+#include "ui/logout-view.h"
 
 #include "file-browser-manager.h"
 #include "file-browser-dialog.h"
@@ -39,7 +40,8 @@ enum {
     INDEX_LOADING_VIEW = 0,
     INDEX_TABLE_VIEW,
     INDEX_LOADING_FAILED_VIEW,
-    INDEX_EMPTY_VIEW
+    INDEX_EMPTY_VIEW,
+    INDEX_RELOGIN_VIEW
 };
 
 const char *kLoadingFailedLabelName = "LoadingFailedText";
@@ -115,6 +117,8 @@ FileBrowserDialog::FileBrowserDialog(const Account &account, const ServerRepo& r
     createEmptyView();
     createFileTable();
 
+    relogin_view_ = new LogoutView;
+
     QWidget* widget = new QWidget;
     widget->setObjectName("mainWidget");
     QVBoxLayout* layout = new QVBoxLayout;
@@ -133,7 +137,10 @@ FileBrowserDialog::FileBrowserDialog(const Account &account, const ServerRepo& r
     stack_->insertWidget(INDEX_TABLE_VIEW, table_view_);
     stack_->insertWidget(INDEX_LOADING_FAILED_VIEW, loading_failed_view_);
     stack_->insertWidget(INDEX_EMPTY_VIEW, empty_view_);
+    stack_->insertWidget(INDEX_RELOGIN_VIEW, relogin_view_);
     stack_->setContentsMargins(0, 0, 0, 0);
+    stack_->installEventFilter(this);
+    stack_->setAcceptDrops(true);
 
     vlayout->addWidget(toolbar_);
     vlayout->addWidget(stack_);
@@ -166,6 +173,11 @@ FileBrowserDialog::FileBrowserDialog(const Account &account, const ServerRepo& r
             this, SLOT(onCancelDownload(const SeafDirent&)));
     connect(table_view_, SIGNAL(syncSubdirectory(const QString&)),
             this, SLOT(onGetSyncSubdirectory(const QString &)));
+    connect(table_view_, SIGNAL(deleteLocalVersion(const SeafDirent&)),
+            this, SLOT(onDeleteLocalVersion(const SeafDirent&)));
+    connect(table_view_, SIGNAL(localVersionSaveAs(const SeafDirent&)),
+            this, SLOT(onLocalVersionSaveAs(const SeafDirent&)));
+
 
     //dirents <--> data_mgr_
     connect(data_mgr_, SIGNAL(getDirentsSuccess(bool, const QList<SeafDirent>&)),
@@ -229,7 +241,12 @@ FileBrowserDialog::FileBrowserDialog(const Account &account, const ServerRepo& r
     connect(AutoUpdateManager::instance(), SIGNAL(fileUpdated(const QString&, const QString&)),
             this, SLOT(onFileAutoUpdated(const QString&, const QString&)));
 
+    AccountManager* account_mgr = seafApplet->accountManager();
+    connect(account_mgr, SIGNAL(accountInfoUpdated(const Account&)), this,
+            SLOT(onAccountInfoUpdated()));
+
     QTimer::singleShot(0, this, SLOT(init()));
+
 }
 
 FileBrowserDialog::~FileBrowserDialog()
@@ -301,14 +318,9 @@ void FileBrowserDialog::createStatusBar()
     connect(upload_file_action_, SIGNAL(triggered()), this, SLOT(chooseFileToUpload()));
     upload_menu_->addAction(upload_file_action_);
 
-    // Submenu's Action 2: Upload directory (only pro version's server supports it)
-    if (account_.isPro()) {
-        upload_directory_action_ = new QAction(tr("Upload a directory"), upload_menu_);
-        connect(upload_directory_action_, SIGNAL(triggered()), this, SLOT(chooseDirectoryToUpload()));
-        upload_menu_->addAction(upload_directory_action_);
-    } else {
-        upload_directory_action_ = NULL;
-    }
+    upload_directory_action_ = new QAction(tr("Upload a directory"), upload_menu_);
+    connect(upload_directory_action_, SIGNAL(triggered()), this, SLOT(chooseDirectoryToUpload()));
+    upload_menu_->addAction(upload_directory_action_);
 
     // Submenu's Action 3: Make a new directory
     mkdir_action_ = new QAction(tr("Create a folder"), upload_menu_);
@@ -351,9 +363,57 @@ void FileBrowserDialog::createFileTable()
     table_view_ = new FileTableView(this);
     table_model_ = new FileTableModel(this);
     table_view_->setModel(table_model_);
+}
 
-    connect(table_view_, SIGNAL(dropFile(const QStringList&)),
-            this, SLOT(uploadOrUpdateMutipleFile(const QStringList&)));
+bool FileBrowserDialog::handleDragDropEvent(QObject *obj, QEvent *event)
+{
+    if (event->type() == QEvent::DragEnter) {
+        QDragEnterEvent* ev = (QDragEnterEvent*)event;
+        // only handle external source currently
+        if(ev->source() != NULL)
+            return false;
+        // Otherwise it might be a MoveAction which is unacceptable
+        ev->setDropAction(Qt::CopyAction);
+        // trivial check
+        if(ev->mimeData()->hasFormat("text/uri-list"))
+            ev->accept();
+        return true;
+    }
+    else if (event->type() == QEvent::Drop) {
+        QDropEvent* ev = (QDropEvent*)event;
+        // only handle external source currently
+        if(ev->source() != NULL)
+            return false;
+
+        QList<QUrl> urls = ev->mimeData()->urls();
+
+        if(urls.isEmpty())
+            return false;
+
+        QStringList paths;
+        Q_FOREACH(const QUrl& url, urls)
+        {
+            QString path = url.toLocalFile();
+#if defined(Q_OS_MAC) && (QT_VERSION <= QT_VERSION_CHECK(5, 4, 0))
+            path = utils::mac::fix_file_id_url(path);
+#endif
+            if(path.isEmpty())
+                continue;
+            paths.push_back(path);
+        }
+
+        ev->accept();
+
+        if (current_readonly_) {
+            seafApplet->warningBox(tr("You do not have permission to upload to this folder"), this);
+        } else {
+            uploadOrUpdateMutipleFile(paths);
+        }
+        return true;
+    }
+    else {
+        return QObject::eventFilter(obj, event);
+    }
 }
 
 bool FileBrowserDialog::eventFilter(QObject *obj, QEvent *event)
@@ -381,7 +441,13 @@ bool FileBrowserDialog::eventFilter(QObject *obj, QEvent *event)
                                           "margin-right: 15px;}");
             return true;
         }
+    } else if (obj == stack_) {
+        if (stack_->currentIndex() == INDEX_EMPTY_VIEW ||
+            stack_->currentIndex() == INDEX_TABLE_VIEW) {
+            return handleDragDropEvent(obj, event);
+        }
     }
+
     return QObject::eventFilter(obj, event);
 }
 
@@ -432,7 +498,11 @@ void FileBrowserDialog::onGetDirentsSuccess(bool current_readonly, const QList<S
 
 void FileBrowserDialog::onGetDirentsFailed(const ApiError& error)
 {
-    stack_->setCurrentIndex(INDEX_LOADING_FAILED_VIEW);
+    if (error.httpErrorCode() == 401) {
+        stack_->setCurrentIndex(INDEX_RELOGIN_VIEW);
+    } else {
+        stack_->setCurrentIndex(INDEX_LOADING_FAILED_VIEW);
+    }
 }
 
 void FileBrowserDialog::onMkdirButtonClicked()
@@ -475,9 +545,10 @@ void FileBrowserDialog::createLoadingFailedView()
 
 void FileBrowserDialog::createEmptyView()
 {
-    empty_view_ = new QLabel;
+    empty_view_ = new QLabel(this);
     empty_view_->setText(tr("This folder is empty."));
     empty_view_->setAlignment(Qt::AlignCenter);
+    empty_view_->setStyleSheet("background-color: white;");
 }
 
 void FileBrowserDialog::onDirentClicked(const SeafDirent& dirent)
@@ -565,6 +636,16 @@ void FileBrowserDialog::enterPath(const QString& path)
     // use QUrl::toPercentEncoding if need
     fetchDirents();
 
+    // QHash<QString, AutoUpdateManager::FileStatus> uploads =
+    //     AutoUpdateManager::instance()->getFileStatusForDirectory(
+    //         account_.getSignature(), repo_.id, path);
+    // if (uploads.empty()) {
+    //     printf("no uploads for dir %s\n", toCStr(path));
+    // }
+    // foreach(const QString& key, uploads.keys()) {
+    //     printf("auto upload status: file=\"%s\", uploading=%d\n", toCStr(key), uploads[key]);
+    // }
+
     // current_path should be guaranteed safe to split!
     current_lpath_ = current_path_.split('/');
     // if the last element is empty (i.e. current_path_ ends with an extra "/"),
@@ -663,14 +744,16 @@ void FileBrowserDialog::downloadFile(const QString& path)
     connect(task, SIGNAL(finished(bool)), this, SLOT(onDownloadFinished(bool)));
 }
 
+void FileBrowserDialog::onGetDirentReupload(const SeafDirent& dirent)
+{
+    QString path = ::pathJoin(current_path_, dirent.name);
+    QString local_path = DataManager::getLocalCacheFilePath(repo_.id, path);
+    AutoUpdateManager::instance()->uploadFile(local_path);
+}
+
 void FileBrowserDialog::uploadFile(const QString& path, const QString& name,
                                    bool overwrite)
 {
-    if (QFileInfo(path).isDir() && !account_.isPro()) {
-        seafApplet->warningBox(tr("Feature not supported"), this);
-        return;
-    }
-
     FileUploadTask *task =
       data_mgr_->createUploadTask(repo_.id, current_path_, path, name, overwrite);
     connect(task, SIGNAL(finished(bool)), this, SLOT(onUploadFinished(bool)));
@@ -794,20 +877,27 @@ void FileBrowserDialog::onDownloadFinished(bool success)
 
         if (task->error() == FileNetworkTask::TaskCanceled)
             return;
-  
-        if(task->httpErrorCode() == 404) {
-             _error = tr("File does not exit");
+
+        if (task->httpErrorCode() == 404) {
+            _error = tr("File does not exist");
+        } else if (task->httpErrorCode() == 401) {
+            _error = tr("Authorization expired");
         } else {
-             _error = task->errorString();
+            _error = task->errorString();
         }
         QString msg = tr("Failed to download file: %1").arg(_error);
         seafApplet->warningBox(msg, this);
+
+        if (task->httpErrorCode() == 401) {
+            stack_->setCurrentIndex(INDEX_RELOGIN_VIEW);
+        }
     }
 }
 
 void FileBrowserDialog::onUploadFinished(bool success)
 {
     FileUploadTask *task = qobject_cast<FileUploadTask *>(sender());
+    QString _error;
     if (task == NULL)
         return;
 
@@ -824,8 +914,21 @@ void FileBrowserDialog::onUploadFinished(bool success)
         if (task->error() == FileNetworkTask::TaskCanceled)
             return;
 
-        QString msg = tr("Failed to upload file: %1").arg(task->errorString());
+        if (task->httpErrorCode() == 403) {
+            _error = tr("Permission Error!");
+        } else if (task->httpErrorCode() == 404) {
+            _error = tr("Library/Folder not found.");
+        } else if (task->httpErrorCode() == 401) {
+            _error = tr("Authorization expired");
+        } else {
+            _error = task->errorString();
+        }
+        QString msg = tr("Failed to upload file %1: %2").arg(task->failedPath()).arg(_error);
+        qWarning("failed to upload %s",toCStr(QFileInfo(task->failedPath()).fileName()));
         seafApplet->warningBox(msg, this);
+        if (task->httpErrorCode() == 401) {
+            stack_->setCurrentIndex(INDEX_RELOGIN_VIEW);
+        }
         return;
     }
 
@@ -870,6 +973,11 @@ void FileBrowserDialog::onUploadFinished(bool success)
         else
             table_model_->replaceItem(name, dirent);
     }
+
+    if (stack_->currentIndex() == INDEX_EMPTY_VIEW) {
+        forceRefresh();
+    }
+
     updateFileCount();
 }
 
@@ -921,17 +1029,13 @@ void FileBrowserDialog::goHome()
 
 void FileBrowserDialog::updateTable(const QList<SeafDirent>& dirents)
 {
-    // Commented out because the empty view can't handle file drag & drop events.
-
-    // if (dirents.isEmpty()) {
-    //     stack_->setCurrentIndex(INDEX_EMPTY_VIEW);
-    // } else {
-    //     table_model_->setDirents(dirents);
-    //     stack_->setCurrentIndex(INDEX_TABLE_VIEW);
-    // }
-
-    table_model_->setDirents(dirents);
-    stack_->setCurrentIndex(INDEX_TABLE_VIEW);
+    if (dirents.isEmpty()) {
+        table_model_->setDirents(QList<SeafDirent>());
+        stack_->setCurrentIndex(INDEX_EMPTY_VIEW);
+    } else {
+        table_model_->setDirents(dirents);
+        stack_->setCurrentIndex(INDEX_TABLE_VIEW);
+    }
 
     if (!forward_history_.empty()) {
         forward_button_->setEnabled(true);
@@ -1131,6 +1235,10 @@ void FileBrowserDialog::onDirentRemoveSuccess(const QString& path)
         return;
     table_model_->removeItemNamed(name);
     updateFileCount();
+
+    if (table_model_->rowCount() == 0) {
+        stack_->setCurrentIndex(INDEX_EMPTY_VIEW);
+    }
 }
 
 void FileBrowserDialog::onDirentRemoveFailed(const ApiError&error)
@@ -1149,6 +1257,10 @@ void FileBrowserDialog::onDirentsRemoveSuccess(const QString& parent_path,
         table_model_->removeItemNamed(name);
     }
     updateFileCount();
+
+    if (table_model_->rowCount() == 0) {
+        stack_->setCurrentIndex(INDEX_EMPTY_VIEW);
+    }
 }
 
 void FileBrowserDialog::onDirentsRemoveFailed(const ApiError&error)
@@ -1266,6 +1378,43 @@ void FileBrowserDialog::onGetSyncSubdirectory(const QString &folder_name)
     data_mgr_->createSubrepo(folder_name, repo_.id, ::pathJoin(current_path_, folder_name));
 }
 
+void FileBrowserDialog::onDeleteLocalVersion(const SeafDirent &dirent)
+{
+    QString fpath = ::pathJoin(current_path_, dirent.name);
+    QString cached_file = data_mgr_->getLocalCachedFile(repo_.id, fpath, dirent.id);
+    if (!cached_file.isEmpty() && QFileInfo(cached_file).exists()) {
+        QFile::remove(cached_file);
+        return;
+    }
+}
+
+void FileBrowserDialog::onLocalVersionSaveAs(const SeafDirent &dirent)
+{
+    static QDir download_dir(defaultDownloadDir());
+    if (!download_dir.exists())
+       download_dir = QDir::home();
+
+     QString fpath = ::pathJoin(current_path_, dirent.name);
+     QString cached_file = data_mgr_->getLocalCachedFile(repo_.id, fpath, dirent.id);
+     if (!cached_file.isEmpty() && QFileInfo(cached_file).exists()) {
+         QString local_path = QFileDialog::getSaveFileName(this, tr("Enter name of file to save to..."), download_dir.filePath(dirent.name));
+         QFile::copy(cached_file, local_path);
+         return;
+     }
+}
+
+void FileBrowserDialog::onOpenLocalCacheFolder()
+{
+     QString folder =
+             ::pathJoin(data_mgr_->getRepoCacheFolder(repo_.id), current_path_);
+     if (!::createDirIfNotExists(folder)) {
+         seafApplet->warningBox(tr("Unable to create cache folder"), this);
+         return;
+     }
+     if (!QDesktopServices::openUrl(QUrl::fromLocalFile(folder)))
+         seafApplet->warningBox(tr("Unable to open cache folder"), this);
+}
+
 void FileBrowserDialog::onCreateSubrepoSuccess(const ServerRepo &repo)
 {
     // if we have not synced before
@@ -1316,3 +1465,8 @@ void FileBrowserDialog::fixUploadButtonStyle(bool highlighted)
                                       "margin-left: 15px;}");
     }
 }
+
+void FileBrowserDialog::onAccountInfoUpdated()
+{
+    forceRefresh();
+}
