@@ -50,6 +50,8 @@ public:
 
     // only accept path ends with "/"
     void enterPath(const QString& path);
+    void onGetDirentReupload(const SeafDirent& dirent);
+    void onOpenLocalCacheFolder();
 
     friend class FileTableView;
     friend class FileTableModel;
@@ -95,6 +97,8 @@ private slots:
     void onGetSyncSubdirectory(const QString &folder_name);
     void onCancelDownload(const SeafDirent& dirent);
 
+    void onDeleteLocalVersion(const SeafDirent& dirent);
+    void onLocalVersionSaveAs(const SeafDirent& dirent);
     void onDirectoryCreateSuccess(const QString& path);
     void onDirectoryCreateFailed(const ApiError& error);
     void onFileLockSuccess(const QString& path, bool lock);
@@ -125,6 +129,8 @@ private slots:
     void fixUploadButtonNonHighlightStyle();
     void fixUploadButtonHighlightStyle();
 
+    void onAccountInfoUpdated();
+
 private:
     Q_DISABLE_COPY(FileBrowserDialog)
 
@@ -155,6 +161,7 @@ private:
     bool setPasswordAndRetry(FileNetworkTask *task);
 
     bool eventFilter(QObject *obj, QEvent *event);
+    bool handleDragDropEvent(QObject *obj, QEvent *event);
 
     const Account account_;
     const ServerRepo repo_;
@@ -201,6 +208,7 @@ private:
     QStackedWidget *stack_;
     QWidget *loading_view_;
     QLabel *loading_failed_view_;
+    QWidget *relogin_view_;
     QLabel *empty_view_;
     FileTableView *table_view_;
     FileTableModel *table_model_;
@@ -208,5 +216,4 @@ private:
     DataManager *data_mgr_;
 };
 
-
 #endif  // SEAFILE_CLIENT_FILE_BROWSER_DIALOG_H
