@@ -60,7 +60,8 @@ ReliablePostFileTask::ReliablePostFileTask(const Account &account,
     current_chunk_size_ = 0;
     done_ = 0;
     total_size_ = 0;
-    resumable_ = true;
+    // Chunked uploading is disabled for updating an existing file
+    resumable_ = use_upload;
 }
 
 ReliablePostFileTask::ReliablePostFileTask(const QUrl &url,
@@ -138,16 +139,22 @@ void ReliablePostFileTask::checkUploadedBytes()
     file_uploaded_bytes_req_.reset(
         new GetFileUploadedBytesRequest(account_, repo_id_, parent_dir_, name_));
 
-    connect(file_uploaded_bytes_req_.data(), SIGNAL(success(quint64)),
-            this, SLOT(onGetFileUploadedBytesSuccess(quint64)));
+    connect(file_uploaded_bytes_req_.data(), SIGNAL(success(bool, quint64)),
+            this, SLOT(onGetFileUploadedBytesSuccess(bool, quint64)));
     connect(file_uploaded_bytes_req_.data(), SIGNAL(failed(const ApiError&)),
             this, SLOT(onGetFileUploadedBytesFailed(const ApiError&)));
     file_uploaded_bytes_req_->send();
 }
 
-void ReliablePostFileTask::onGetFileUploadedBytesSuccess(quint64 uploaded_bytes)
+void ReliablePostFileTask::onGetFileUploadedBytesSuccess(bool support_chunked_uploading, quint64 uploaded_bytes)
 {
-    current_offset_ = uploaded_bytes;
+    if (!support_chunked_uploading) {
+        resumable_ = false;
+        current_offset_ = 0;
+    } else {
+        current_offset_ = uploaded_bytes;
+    }
+
     startPostFileTask();
 }
 
@@ -226,6 +233,10 @@ void ReliablePostFileTask::onPostFileTaskFinished(bool result)
         }
     }
 
+    http_error_code_ = task_->httpErrorCode();
+    error_string_ = task_->errorString();
+    error_ = task_->error();
+
     if (!useResumableUpload()) {
         // Simple upload
         emit finished(result);
