@@ -88,9 +88,7 @@ FileNetworkTask::FileNetworkTask(const Account& account,
       path_(path),
       local_path_(local_path),
       canceled_(false),
-      progress_(0, 0),
-      __shared_ptr(this, &QObject::deleteLater),
-      __weak_ptr(__shared_ptr)
+      progress_(0, 0)
 {
     fileserver_task_ = NULL;
     get_link_req_ = NULL;
@@ -188,6 +186,7 @@ void FileNetworkTask::onFileServerTaskFinished(bool success)
         error_ = fileserver_task_->error();
         error_string_ = fileserver_task_->errorString();
         http_error_code_ = fileserver_task_->httpErrorCode();
+        failed_path_ = fileserver_task_->failedPath();
     }
     oid_ = fileserver_task_->oid();
     onFinished(success);
@@ -196,7 +195,7 @@ void FileNetworkTask::onFileServerTaskFinished(bool success)
 void FileNetworkTask::onFinished(bool success)
 {
     emit finished(success);
-    __shared_ptr.clear();
+    deleteLater();
 }
 
 void FileNetworkTask::onGetLinkFailed(const ApiError& error)
@@ -713,6 +712,10 @@ void PostFilesTask::onPostTaskFinished(bool success)
         error_string_ = task_->errorString();
         http_error_code_ = task_->httpErrorCode();
         progress_update_timer_->stop();
+
+        const QString& file_path = names_[current_num_];
+        QString file_name = QFileInfo(file_path).fileName();
+        failed_path_ = file_name;
         emit finished(false);
         return;
     }
