@@ -3,7 +3,6 @@
 
 #include <QObject>
 #include <QUrl>
-#include <QSharedPointer>
 #include <QMutex>
 
 #include "api/server-repo.h"
@@ -78,6 +77,7 @@ public:
     const Account& account() const { return account_; };
     QString path() const { return path_; };
     QString localFilePath() const { return local_path_; }
+    QString failedPath() const { return failed_path_; }
     QString fileName() const;
     QString oid() const { return oid_; }
     Progress progress() const { return progress_; };
@@ -93,19 +93,6 @@ public:
     const QString& errorString() const { return error_string_; }
     int httpErrorCode() const { return http_error_code_; }
 
-    // shared_from_this
-    // Usage: (for example, FileDownloadTask)
-    //  FileDownloadTask *task = new FileDownloadTask(...);
-    //  SharedPointer<FileNetworkTask> sharePtr = task->sharedFromThis();
-    //  SharedPointer<FileDownloadTask> sharePtr =
-    //  task->sharedFromThis().objectCast<FileDownloadTask>();
-    //
-    QSharedPointer<FileNetworkTask> sharedFromThis() {
-        QSharedPointer<FileNetworkTask> shared_ptr(__weak_ptr.toStrongRef());
-        __shared_ptr.clear();
-        return shared_ptr;
-    }
-
 public slots:
     virtual void start();
     virtual void cancel();
@@ -134,6 +121,7 @@ protected:
     const QString repo_id_;
     QString path_;
     QString local_path_;
+    QString failed_path_;
     QString oid_;
 
     TaskError error_;
@@ -144,10 +132,6 @@ protected:
     Progress progress_;
 
     static QThread *worker_thread_;
-    // keep a copy of shared_ptr
-    // for we can't get shared_ptr from weak_ptr
-    QSharedPointer<FileNetworkTask> __shared_ptr;
-    QWeakPointer<FileNetworkTask> __weak_ptr;
 };
 
 
@@ -277,6 +261,7 @@ public:
     int httpErrorCode() const { return http_error_code_; }
     virtual const QString& oid() const { return oid_; }
     int retryCount() const { return retry_count_; }
+    const QString& failedPath() const { return failed_path_; }
 
     static void resetQNAM();
 
@@ -328,6 +313,7 @@ protected:
 
     QUrl url_;
     QString local_path_;
+    QString failed_path_;
     QString oid_;
 
     QNetworkReply *reply_;
