@@ -34,6 +34,7 @@
 #include "filebrowser/thumbnail-service.h"
 #include "filebrowser/data-cache.h"
 #include "filebrowser/auto-update-mgr.h"
+#include "filebrowser/data-mgr.h"
 #include "rpc/local-repo.h"
 #include "network-mgr.h"
 #include "server-status-service.h"
@@ -195,9 +196,11 @@ SeafileApplet::SeafileApplet()
       settings_dialog_(new SettingsDialog),
       settings_mgr_(new SettingsManager),
       certs_mgr_(new CertsManager),
+      data_mgr_(new DataManager),
       started_(false),
       in_exit_(false),
-      is_pro_(false)
+      is_pro_(false),
+      about_to_quit_(false)
 {
     tray_icon_ = new SeafileTrayIcon(this);
     connect(qApp, SIGNAL(aboutToQuit()), this, SLOT(onAboutToQuit()));
@@ -219,6 +222,7 @@ SeafileApplet::~SeafileApplet()
 
     delete account_mgr_;
     delete configurator_;
+    delete data_mgr_;
     if (main_win_)
         delete main_win_;
 #if defined(Q_OS_WIN32)
@@ -243,6 +247,8 @@ void SeafileApplet::start()
 
     certs_mgr_->start();
 
+    data_mgr_->start();
+
 #if defined(Q_OS_WIN32)
     QString crash_rpt_path = QDir(configurator_->ccnetDir()).filePath("logs/seafile-crash-report.txt");
     if (!g_setenv ("CRASH_RPT_PATH", toCStr(crash_rpt_path), FALSE))
@@ -259,6 +265,8 @@ void SeafileApplet::start()
         account_mgr_->currentAccount().serverUrl,
         QDir(configurator_->seafileDir()).filePath("system-proxy.txt"));
 
+    FileCache::instance()->start();
+
     //
     // start daemons
     //
@@ -392,6 +400,7 @@ void SeafileApplet::checkInitVDrive()
 
 void SeafileApplet::onAboutToQuit()
 {
+    about_to_quit_ = true;
     tray_icon_->hide();
     if (main_win_) {
         main_win_->writeSettings();
